@@ -3,14 +3,14 @@ use strict;
 use warnings;
 package Email::MIME;
 # ABSTRACT: easy MIME message handling
-$Email::MIME::VERSION = '1.937';
-use Email::Simple 2.206; # header_raw
+$Email::MIME::VERSION = '1.949';
+use Email::Simple 2.212; # nth header value
 use parent qw(Email::Simple);
 
 use Carp ();
 use Email::MessageID;
 use Email::MIME::Creator;
-use Email::MIME::ContentType 1.016; # type/subtype, not discrete/composite
+use Email::MIME::ContentType 1.023; # build_content_type
 use Email::MIME::Encode;
 use Email::MIME::Encodings 1.314;
 use Email::MIME::Header;
@@ -18,6 +18,8 @@ use Email::MIME::Modifier;
 use Encode 1.9801 ();
 use Scalar::Util qw(reftype weaken);
 
+our @CARP_NOT = qw(Email::MIME::ContentType);
+
 #pod =head1 SYNOPSIS
 #pod
 #pod B<Wait!>  Before you read this, maybe you just need L<Email::Stuffer>, which is
@@ -61,7 +63,11 @@ use Scalar::Util qw(reftype weaken);
 #pod   );
 #pod
 #pod   my $email = Email::MIME->create(
-#pod       header_str => [ From => 'casey@geeknest.com' ],
+#pod       header_str => [
+#pod           From => 'casey@geeknest.com',
+#pod           To => [ 'user1@host.com', 'Name <user2@host.com>' ],
+#pod           Cc => Email::Address::XS->new("Display Name \N{U+1F600}", 'user@example.com'),
+#pod       ],
 #pod       parts      => [ @parts ],
 #pod   );
 #pod
@@ -83,7 +89,7 @@ use Scalar::Util qw(reftype weaken);
 #pod   $_->encoding_set( 'base64' ) for $email->parts;
 #pod
 #pod   # Quick multipart creation
-#pod   my $quicky = Email::MIME->create(
+#pod   my $email = Email::MIME->create(
 #pod       header_str => [
 #pod           From => 'my@address',
 #pod           To   => 'your@address',
@@ -119,8 +125,16 @@ my $NO_ENCODE_RE = qr/
 /ix;
 
 sub new {
-  my $self = shift->SUPER::new(@_);
-  $self->{ct} = parse_content_type($self->content_type);
+  my ($class, $text, $arg, @rest) = @_;
+  $arg ||= {};
+
+  my $encode_check = exists $arg->{encode_check}
+                   ? delete $arg->{encode_check}
+                   : Encode::FB_CROAK;
+
+  my $self = shift->SUPER::new($text, $arg, @rest);
+  $self->encode_check_set($encode_check);
+  $self->{ct} = parse_content_type($self->content_type_raw);
   $self->parts;
   return $self;
 }
@@ -141,12 +155,29 @@ sub new {
 #pod
 #pod This method creates a new MIME part. The C<header_str> parameter is a list of
 #pod headers pairs to include in the message. The value for each pair is expected to
-#pod be a text string that will be MIME-encoded as needed.  A similar C<header>
-#pod parameter can be provided in addition to or instead of C<header_str>.  Its
-#pod values will be used verbatim.
+#pod be a text string that will be MIME-encoded as needed.  Alternatively it can be
+#pod an object with C<as_mime_string> method which implements conversion of that
+#pod object to MIME-encoded string.  That object method is called with two named
+#pod input parameters: C<charset> and C<header_name_length>.  It should return
+#pod MIME-encoded representation of the object.  As of 2017-07-25, the
+#pod header-value-as-object code is very young, and may yet change.
+#pod
+#pod In case header name is registered in C<%Email::MIME::Header::header_to_class_map>
+#pod hash then registered class is used for conversion from Unicode string to 8bit
+#pod MIME encoding.  Value can be either string or array reference to strings.
+#pod Object is constructed via method C<from_string> with string value (or values
+#pod in case of array reference) and converted to MIME-encoded string via
+#pod C<as_mime_string> method.
+#pod
+#pod A similar C<header> parameter can be provided in addition to or instead of
+#pod C<header_str>.  Its values will be used verbatim.
 #pod
 #pod C<attributes> is a hash of MIME attributes to assign to the part, and may
-#pod override portions of the header set in the C<header> parameter.
+#pod override portions of the header set in the C<header> parameter. The hash keys
+#pod correspond directly to methods for modifying a message from
+#pod C<Email::MIME::Modifier>. The allowed keys are: content_type, charset, name,
+#pod format, boundary, encoding, disposition, and filename. They will be mapped to
+#pod C<"$attr\_set"> for message modification.
 #pod
 #pod The C<parts> parameter is a list reference containing C<Email::MIME>
 #pod objects. Elements of the C<parts> list can also be a non-reference
@@ -161,12 +192,6 @@ sub new {
 #pod character string to be used as the body.  If you provide a C<body_str>
 #pod parameter, you B<must> provide C<charset> and C<encoding> attributes.
 #pod
-#pod Back to C<attributes>. The hash keys correspond directly to methods or
-#pod modifying a message from C<Email::MIME::Modifier>. The allowed keys are:
-#pod content_type, charset, name, format, boundary, encoding, disposition,
-#pod and filename. They will be mapped to C<"$attr\_set"> for message
-#pod modification.
-#pod
 #pod =cut
 
 my %CT_SETTER = map {; $_ => 1 } qw(
@@ -222,9 +247,15 @@ sub create {
     $CREATOR->_add_to_header(\$header, 'Content-Type' => 'text/plain',);
   }
 
-  my $email = $class->new($header);
+  my %pass_on;
 
-  for my $key (keys %attrs) {
+  if (exists $args{encode_check}) {
+    $pass_on{encode_check} = $args{encode_check};
+  }
+
+  my $email = $class->new($header, \%pass_on);
+
+  for my $key (sort keys %attrs) {
     $email->content_type_attribute_set($key => $attrs{$key});
   }
 
@@ -247,7 +278,7 @@ sub create {
     Carp::confess("body_str was given, but no encoding is defined")
       unless $attrs{encoding};
 
-    my $body_octets = Encode::encode($attrs{charset}, $args{body_str}, 1);
+    my $body_octets = Encode::encode($attrs{charset}, $args{body_str}, $email->encode_check);
     $email->body_set($body_octets);
   }
 
@@ -342,21 +373,29 @@ sub body_str {
     Carp::confess("can't get body as a string for " . $self->content_type);
   }
 
-  my $str = Encode::decode($encoding, $self->body, 1);
+  my $str = Encode::decode($encoding, $self->body, $self->encode_check);
   return $str;
 }
 
+our $MAX_DEPTH = 10;
+
 sub parts_multipart {
   my $self     = shift;
   my $boundary = $self->{ct}->{attributes}->{boundary};
 
+  our $DEPTH ||= 0;
+
+  Carp::croak("attempted to parse a MIME message more than $MAX_DEPTH deep")
+    if $MAX_DEPTH && $DEPTH > $MAX_DEPTH;
+
   # Take a message, join all its lines together.  Now try to Email::MIME->new
   # it with 1.861 or earlier.  Death!  It tries to recurse endlessly on the
   # body, because every time it splits on boundary it gets itself. Obviously
   # that means it's a bogus message, but a mangled result (or exception) is
   # better than endless recursion. -- rjbs, 2008-01-07
   return $self->parts_single_part
-    unless $boundary and $self->body_raw =~ /^--\Q$boundary\E\s*$/sm;
+    unless defined $boundary and length $boundary and
+           $self->body_raw =~ /^--\Q$boundary\E\s*$/sm;
 
   $self->{body_raw} = $self->SUPER::body;
 
@@ -379,7 +418,8 @@ sub parts_multipart {
   for my $bit (@bits) {
     $bit =~ s/\A[\n\r]+//smg;
     $bit =~ s/(?<!\x0d)$self->{mycrlf}\Z//sm;
-    my $email = (ref $self)->new($bit);
+    local $DEPTH = $DEPTH + 1;
+    my $email = (ref $self)->new($bit, { encode_check => $self->encode_check });
     push @parts, $email;
   }
 
@@ -391,6 +431,7 @@ sub parts_multipart {
 sub force_decode_hook { 0 }
 sub decode_hook       { return $_[1] }
 sub content_type      { scalar shift->header("Content-type"); }
+sub content_type_raw  { scalar shift->header_raw("Content-type"); }
 
 sub debug_structure {
   my ($self, $level) = @_;
@@ -409,10 +450,7 @@ sub filename {
   return $gcache{$self} if exists $gcache{$self};
 
   my $dis = $self->header("Content-Disposition") || '';
-  my $attrs
-    = $dis =~ s/^.*?;//
-    ? Email::MIME::ContentType::_parse_attributes($dis)
-    : {};
+  my $attrs = parse_content_disposition($dis)->{attributes};
   my $name = $attrs->{filename}
     || $self->{ct}{attributes}{name};
   return $name if $name or !$force;
@@ -448,6 +486,11 @@ sub header_str_pairs {
   $self->header_obj->header_str_pairs(@_);
 }
 
+sub header_as_obj {
+  my $self = shift;
+  $self->header_obj->header_as_obj(@_);
+}
+
 #pod =method content_type_set
 #pod
 #pod   $email->content_type_set( 'text/html' );
@@ -508,7 +551,7 @@ sub boundary_set {
   my ($self, $value) = @_;
   my $ct_header = parse_content_type($self->header('Content-Type'));
 
-  if ($value) {
+  if (defined $value and length $value) {
     $ct_header->{attributes}->{boundary} = $value;
   } else {
     delete $ct_header->{attributes}->{boundary};
@@ -542,6 +585,42 @@ sub content_type_attribute_set {
   $self->_compose_content_type($ct_header);
 }
 
+#pod =method encode_check
+#pod
+#pod =method encode_check_set
+#pod
+#pod   $email->encode_check;
+#pod   $email->encode_check_set(0);
+#pod   $email->encode_check_set(Encode::FB_DEFAULT);
+#pod
+#pod Gets/sets the current C<encode_check> setting (default: I<FB_CROAK>).
+#pod This is the parameter passed to L<Encode/"decode"> and L<Encode/"encode">
+#pod when C<body_str()>, C<body_str_set()>, and C<create()> are called.
+#pod
+#pod With the default setting, Email::MIME may crash if the claimed charset
+#pod of a body does not match its contents (for example - utf8 data in a
+#pod text/plain; charset=us-ascii message).
+#pod
+#pod With an C<encode_check> of 0, the unrecognized bytes will instead be
+#pod replaced with the C<REPLACEMENT CHARACTER> (U+0FFFD), and may end up
+#pod as either that or question marks (?).
+#pod
+#pod See L<Encode/"Handling Malformed Data"> for more information.
+#pod
+#pod =cut
+
+sub encode_check {
+  my ($self) = @_;
+
+  return $self->{encode_check};
+}
+
+sub encode_check_set {
+  my ($self, $val) = @_;
+
+  return $self->{encode_check} = $val;
+}
+
 #pod =method encoding_set
 #pod
 #pod   $email->encoding_set( 'base64' );
@@ -623,7 +702,7 @@ sub body_str_set {
   Carp::confess("body_str was given, but no charset is defined")
     unless my $charset = $ct->{attributes}{charset};
 
-  my $body_octets = Encode::encode($charset, $body_str, 1);
+  my $body_octets = Encode::encode($charset, $body_str, $self->encode_check);
   $self->body_set($body_octets);
 }
 
@@ -658,20 +737,14 @@ sub disposition_set {
 sub filename_set {
   my ($self, $filename) = @_;
   my $dis_header = $self->header('Content-Disposition');
-  my ($disposition, $attrs);
+  my ($disposition, $attrs) = ('inline', {});
   if ($dis_header) {
-    ($disposition) = ($dis_header =~ /^([^;]+)/);
-    $dis_header =~ s/^$disposition(?:;\s*)?//;
-    $attrs = Email::MIME::ContentType::_parse_attributes($dis_header) || {};
+    my $struct = parse_content_disposition($dis_header);
+    $disposition = $struct->{type};
+    $attrs = $struct->{attributes};
   }
   $filename ? $attrs->{filename} = $filename : delete $attrs->{filename};
-  $disposition ||= 'inline';
-
-  my $dis = $disposition;
-  while (my ($attr, $val) = each %{$attrs}) {
-    $dis .= qq[; $attr="$val"];
-  }
-
+  my $dis = build_content_disposition({type => $disposition, attributes => $attrs});
   $self->header_raw_set('Content-Disposition' => $dis);
 }
 
@@ -695,15 +768,22 @@ sub parts_set {
   if (@{$parts} > 1 or $ct_header->{type} eq 'multipart') {
 
     # setup multipart
-    $ct_header->{attributes}->{boundary} ||= Email::MessageID->new->user;
+    $ct_header->{attributes}->{boundary} = Email::MessageID->new->user
+      unless defined $ct_header->{attributes}->{boundary} and length $ct_header->{attributes}->{boundary};
     my $bound = $ct_header->{attributes}->{boundary};
     foreach my $part (@{$parts}) {
       $body .= "$self->{mycrlf}--$bound$self->{mycrlf}";
       $body .= $part->as_string;
     }
     $body .= "$self->{mycrlf}--$bound--$self->{mycrlf}";
-    @{$ct_header}{qw[type subtype]} = qw[multipart mixed]
-      unless grep { $ct_header->{type} eq $_ } qw[multipart message];
+
+    unless (grep { $ct_header->{type} eq $_ } qw[multipart message]) {
+      if (scalar $self->header('Content-Type')) {
+        Carp::carp("replacing non-multipart type ($ct_header->{type}/$ct_header->{subtype}) with multipart/mixed");
+      }
+      @{$ct_header}{qw[type subtype]} = qw[multipart mixed];
+    }
+
     $self->encoding_set('7bit');
     delete $ct_header->{attributes}{charset};
   } elsif (@$parts == 1) {  # setup singlepart
@@ -712,7 +792,11 @@ sub parts_set {
     my $from_ct = parse_content_type($parts->[0]->header('Content-Type'));
     @{$ct_header}{qw[type subtype]} = @{ $from_ct }{qw[type subtype]};
 
-    $ct_header->{attributes}{charset} = $from_ct->{attributes}{charset};
+    if (exists $from_ct->{attributes}{charset}) {
+      $ct_header->{attributes}{charset} = $from_ct->{attributes}{charset};
+    } else {
+      delete $ct_header->{attributes}{charset};
+    }
 
     $self->encoding_set($parts->[0]->header('Content-Transfer-Encoding'));
     delete $ct_header->{attributes}->{boundary};
@@ -806,11 +890,7 @@ sub walk_parts {
 
 sub _compose_content_type {
   my ($self, $ct_header) = @_;
-  my $ct = join q{/}, @{$ct_header}{qw[type subtype]};
-  for my $attr (sort keys %{ $ct_header->{attributes} }) {
-    next unless defined (my $value = $ct_header->{attributes}{$attr});
-    $ct .= qq[; $attr="$value"];
-  }
+  my $ct = build_content_type({type => $ct_header->{type}, subtype => $ct_header->{subtype}, attributes => $ct_header->{attributes}});
   $self->header_raw_set('Content-Type' => $ct);
   $self->{ct} = $ct_header;
 }
@@ -860,7 +940,7 @@ Email::MIME - easy MIME message handling
 
 =head1 VERSION
 
-version 1.937
+version 1.949
 
 =head1 SYNOPSIS
 
@@ -905,7 +985,11 @@ by all means keep reading.
   );
 
   my $email = Email::MIME->create(
-      header_str => [ From => 'casey@geeknest.com' ],
+      header_str => [
+          From => 'casey@geeknest.com',
+          To => [ 'user1@host.com', 'Name <user2@host.com>' ],
+          Cc => Email::Address::XS->new("Display Name \N{U+1F600}", 'user@example.com'),
+      ],
       parts      => [ @parts ],
   );
 
@@ -927,7 +1011,7 @@ by all means keep reading.
   $_->encoding_set( 'base64' ) for $email->parts;
 
   # Quick multipart creation
-  my $quicky = Email::MIME->create(
+  my $email = Email::MIME->create(
       header_str => [
           From => 'my@address',
           To   => 'your@address',
@@ -969,12 +1053,29 @@ very long. Added to that, you have:
 
 This method creates a new MIME part. The C<header_str> parameter is a list of
 headers pairs to include in the message. The value for each pair is expected to
-be a text string that will be MIME-encoded as needed.  A similar C<header>
-parameter can be provided in addition to or instead of C<header_str>.  Its
-values will be used verbatim.
+be a text string that will be MIME-encoded as needed.  Alternatively it can be
+an object with C<as_mime_string> method which implements conversion of that
+object to MIME-encoded string.  That object method is called with two named
+input parameters: C<charset> and C<header_name_length>.  It should return
+MIME-encoded representation of the object.  As of 2017-07-25, the
+header-value-as-object code is very young, and may yet change.
+
+In case header name is registered in C<%Email::MIME::Header::header_to_class_map>
+hash then registered class is used for conversion from Unicode string to 8bit
+MIME encoding.  Value can be either string or array reference to strings.
+Object is constructed via method C<from_string> with string value (or values
+in case of array reference) and converted to MIME-encoded string via
+C<as_mime_string> method.
+
+A similar C<header> parameter can be provided in addition to or instead of
+C<header_str>.  Its values will be used verbatim.
 
 C<attributes> is a hash of MIME attributes to assign to the part, and may
-override portions of the header set in the C<header> parameter.
+override portions of the header set in the C<header> parameter. The hash keys
+correspond directly to methods for modifying a message from
+C<Email::MIME::Modifier>. The allowed keys are: content_type, charset, name,
+format, boundary, encoding, disposition, and filename. They will be mapped to
+C<"$attr\_set"> for message modification.
 
 The C<parts> parameter is a list reference containing C<Email::MIME>
 objects. Elements of the C<parts> list can also be a non-reference
@@ -989,12 +1090,6 @@ If C<body_str> is given instead of C<body> or C<parts>, it is assumed to be a
 character string to be used as the body.  If you provide a C<body_str>
 parameter, you B<must> provide C<charset> and C<encoding> attributes.
 
-Back to C<attributes>. The hash keys correspond directly to methods or
-modifying a message from C<Email::MIME::Modifier>. The allowed keys are:
-content_type, charset, name, format, boundary, encoding, disposition,
-and filename. They will be mapped to C<"$attr\_set"> for message
-modification.
-
 =head2 content_type_set
 
   $email->content_type_set( 'text/html' );
@@ -1019,6 +1114,28 @@ These four methods modify common C<Content-Type> attributes. If set to
 C<undef>, the attribute is removed. All other C<Content-Type> header
 information is preserved when modifying an attribute.
 
+=head2 encode_check
+
+=head2 encode_check_set
+
+  $email->encode_check;
+  $email->encode_check_set(0);
+  $email->encode_check_set(Encode::FB_DEFAULT);
+
+Gets/sets the current C<encode_check> setting (default: I<FB_CROAK>).
+This is the parameter passed to L<Encode/"decode"> and L<Encode/"encode">
+when C<body_str()>, C<body_str_set()>, and C<create()> are called.
+
+With the default setting, Email::MIME may crash if the claimed charset
+of a body does not match its contents (for example - utf8 data in a
+text/plain; charset=us-ascii message).
+
+With an C<encode_check> of 0, the unrecognized bytes will instead be
+replaced with the C<REPLACEMENT CHARACTER> (U+0FFFD), and may end up
+as either that or question marks (?).
+
+See L<Encode/"Handling Malformed Data"> for more information.
+
 =head2 encoding_set
 
   $email->encoding_set( 'base64' );
@@ -1118,6 +1235,9 @@ This behaves like C<header_raw_set>, but expects Unicode (character) strings as
 the values to set, rather than pre-encoded byte strings.  It will encode them
 as MIME encoded-words if they contain any control or 8-bit characters.
 
+Alternatively, values can be objects with C<as_mime_string> method.  Same as in
+method C<create>.
+
 =head2 header_str_pairs
 
   my @pairs = $email->header_str_pairs;
@@ -1126,6 +1246,23 @@ This method behaves like C<header_raw_pairs>, returning a list of field
 name/value pairs, but the values have been decoded to character strings, when
 possible.
 
+=head2 header_as_obj
+
+  my $first_obj = $email->header_as_obj($field);
+  my $nth_obj   = $email->header_as_obj($field, $index);
+  my @all_objs  = $email->header_as_obj($field);
+
+  my $nth_obj_of_class  = $email->header_as_obj($field, $index, $class);
+  my @all_objs_of_class = $email->header_as_obj($field, undef, $class);
+
+This method returns an object representation of the header value.  It instances
+new object via method C<from_mime_string> of specified class.  Input argument
+for that class method is list of the raw MIME-encoded values.  If class argument
+is not specified then class name is taken from the hash
+C<%Email::MIME::Header::header_to_class_map> via key field.  Use class method
+C<< Email::MIME::Header->set_class_for_header($class, $field) >> for adding new
+mapping.
+
 =head2 parts
 
 This returns a list of C<Email::MIME> objects reflecting the parts of the
@@ -1201,6 +1338,12 @@ For example:
     + text/plain
     + text/html
 
+=head1 CONFIGURATION
+
+The variable C<$Email::MIME::MAX_DEPTH> is the maximum depth of parts that will
+be processed.  It defaults to 10, already higher than legitimate mail is ever
+likely to be.  This value may go up over time as the parser is improved.
+
 =head1 TODO
 
 All of the Email::MIME-specific guts should move to a single entry on the
@@ -1236,7 +1379,7 @@ Simon Cozens <simon@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Alex Vandiver Arthur Axel 'fREW' Schmidt Brian Cassidy David Steinbrunner Dotan Dimet Geraint Edwards Jesse Luehrs Kurt Anderson Lance A. Brown Matthew Horsfall (alh) memememomo Shawn Sorichetti Tomohiro Hosaka
+=for stopwords Alex Vandiver Anirvan Chatterjee Arthur Axel 'fREW' Schmidt Brian Cassidy Damian Lukowski Dan Book David Steinbrunner Dotan Dimet dxdc Eric Wong Geraint Edwards Jesse Luehrs Kurt Anderson Lance A. Brown Matthew Horsfall (alh) memememomo Michael McClimon Pali Ricardo Signes Shawn Sorichetti Tomohiro Hosaka
 
 =over 4
 
@@ -1246,6 +1389,10 @@ Alex Vandiver <alexmv@mit.edu>
 
 =item *
 
+Anirvan Chatterjee <anirvan@users.noreply.github.com>
+
+=item *
+
 Arthur Axel 'fREW' Schmidt <frioux@gmail.com>
 
 =item *
@@ -1254,6 +1401,14 @@ Brian Cassidy <bricas@cpan.org>
 
 =item *
 
+Damian Lukowski <damian.lukowski@credativ.de>
+
+=item *
+
+Dan Book <grinnz@gmail.com>
+
+=item *
+
 David Steinbrunner <dsteinbrunner@pobox.com>
 
 =item *
@@ -1262,6 +1417,14 @@ Dotan Dimet <dotan@corky.net>
 
 =item *
 
+dxdc <dan@element26.net>
+
+=item *
+
+Eric Wong <e@80x24.org>
+
+=item *
+
 Geraint Edwards <gedge-oss@yadn.org>
 
 =item *
@@ -1286,6 +1449,18 @@ memememomo <memememomo@gmail.com>
 
 =item *
 
+Michael McClimon <michael@mcclimon.org>
+
+=item *
+
+Pali <pali@cpan.org>
+
+=item *
+
+Ricardo Signes <rjbs@semiotic.systems>
+
+=item *
+
 Shawn Sorichetti <ssoriche@coloredblocks.com>
 
 =item *
@@ -1322,6 +1497,9 @@ __END__
 #pod the values to set, rather than pre-encoded byte strings.  It will encode them
 #pod as MIME encoded-words if they contain any control or 8-bit characters.
 #pod
+#pod Alternatively, values can be objects with C<as_mime_string> method.  Same as in
+#pod method C<create>.
+#pod
 #pod =method header_str_pairs
 #pod
 #pod   my @pairs = $email->header_str_pairs;
@@ -1330,6 +1508,23 @@ __END__
 #pod name/value pairs, but the values have been decoded to character strings, when
 #pod possible.
 #pod
+#pod =method header_as_obj
+#pod
+#pod   my $first_obj = $email->header_as_obj($field);
+#pod   my $nth_obj   = $email->header_as_obj($field, $index);
+#pod   my @all_objs  = $email->header_as_obj($field);
+#pod
+#pod   my $nth_obj_of_class  = $email->header_as_obj($field, $index, $class);
+#pod   my @all_objs_of_class = $email->header_as_obj($field, undef, $class);
+#pod
+#pod This method returns an object representation of the header value.  It instances
+#pod new object via method C<from_mime_string> of specified class.  Input argument
+#pod for that class method is list of the raw MIME-encoded values.  If class argument
+#pod is not specified then class name is taken from the hash
+#pod C<%Email::MIME::Header::header_to_class_map> via key field.  Use class method
+#pod C<< Email::MIME::Header->set_class_for_header($class, $field) >> for adding new
+#pod mapping.
+#pod
 #pod =method parts
 #pod
 #pod This returns a list of C<Email::MIME> objects reflecting the parts of the
@@ -1405,6 +1600,12 @@ __END__
 #pod     + text/plain
 #pod     + text/html
 #pod
+#pod =head1 CONFIGURATION
+#pod
+#pod The variable C<$Email::MIME::MAX_DEPTH> is the maximum depth of parts that will
+#pod be processed.  It defaults to 10, already higher than legitimate mail is ever
+#pod likely to be.  This value may go up over time as the parser is improved.
+#pod
 #pod =head1 TODO
 #pod
 #pod All of the Email::MIME-specific guts should move to a single entry on the
